
# q-expansion of newform 6900.2.f.m, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6900
weight = 2
poly_data = [9, 0, 7, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 4, 0, 1], 3], [[4, 0, 1, 0], 1]]

hecke_ring_character_values = [[3451, [1, 0, 0, 0]], [4601, [1, 0, 0, 0]], [277, [-1, 0, 0, 0]], [1201, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, -1, 0], [0, 0, 0, 0], [0, 1, 3, 0], [0, 0, 0, 1], [0, 1, -1, 0], [0, 0, 3, 0], [-2, 0, 0, -2], [0, 0, 1, 0], [-6, 0, 0, 2], [2, 0, 0, -5], [0, 2, 1, 0], [3, 0, 0, -2], [0, 0, 1, 0], [0, 0, -3, 0], [0, -3, -9, 0], [0, 0, 0, -5], [-4, 0, 0, 3], [0, 1, -12, 0], [-6, 0, 0, 7], [0, 4, 5, 0], [-5, 0, 0, 4], [0, -2, -11, 0], [-9, 0, 0, 6], [0, -9, -7, 0], [-3, 0, 0, -3], [0, 2, -6, 0], [0, -6, -3, 0], [7, 0, 0, 0], [0, 9, 6, 0], [0, 0, 2, 0], [9, 0, 0, -4], [0, -4, 2, 0], [-2, 0, 0, -5], [3, 0, 0, 4], [-13, 0, 0, 6], [0, 0, -4, 0], [0, -6, -11, 0], [0, 5, -4, 0], [0, -5, -17, 0], [-6, 0, 0, 11], [2, 0, 0, -10], [3, 0, 0, 6], [0, 0, -11, 0], [0, 6, 12, 0], [-20, 0, 0, 6], [-16, 0, 0, 2], [0, 1, -16, 0], [0, 4, 7, 0], [1, 0, 0, 6], [0, 3, -15, 0], [0, 0, 0, 2], [5, 0, 0, -5], [-6, 0, 0, -7], [0, 7, 22, 0], [0, -3, 0, 0], [6, 0, 0, -14], [-13, 0, 0, 4], [0, 3, -4, 0], [-21, 0, 0, 6], [0, 12, -5, 0], [0, 2, -1, 0], [0, -4, -26, 0], [9, 0, 0, -4], [0, 4, -13, 0], [0, -3, 3, 0], [-13, 0, 0, -1], [0, 12, 11, 0], [0, -12, 6, 0], [19, 0, 0, -6], [0, -1, 23, 0], [-21, 0, 0, 4], [0, -4, 7, 0], [0, 2, -27, 0], [-8, 0, 0, -4], [0, 11, 5, 0], [-18, 0, 0, -5], [0, -4, 16, 0], [-24, 0, 0, 1], [10, 0, 0, 6], [-21, 0, 0, 3], [-13, 0, 0, -1], [-15, 0, 0, -8], [0, -2, -1, 0], [7, 0, 0, -8], [0, 4, 22, 0], [12, 0, 0, -15], [0, 8, 22, 0], [0, 0, 0, -9], [0, -11, 5, 0], [0, 18, 0, 0], [-18, 0, 0, -2], [0, 2, 28, 0], [-24, 0, 0, -1], [7, 0, 0, 0], [0, -2, 7, 0], [-6, 0, 0, 13], [-6, 0, 0, -2], [0, -10, -3, 0], [2, 0, 0, 18], [0, 8, -5, 0], [0, 2, 8, 0], [0, 3, 24, 0], [-15, 0, 0, 0], [-13, 0, 0, 0], [0, 3, 11, 0], [0, 1, 4, 0], [0, 0, 3, 0], [-15, 0, 0, -14], [14, 0, 0, -4], [0, 3, 8, 0], [0, 17, -6, 0], [0, 9, 27, 0], [-14, 0, 0, 9], [-7, 0, 0, -1], [24, 0, 0, 11], [0, -3, 19, 0], [0, -16, 2, 0], [0, 16, 25, 0], [-15, 0, 0, -7], [20, 0, 0, -7], [0, 0, -8, 0], [0, 5, -40, 0], [0, 6, 3, 0], [-37, 0, 0, 0], [-9, 0, 0, 10], [16, 0, 0, 6], [-3, 0, 0, 2], [0, 8, 10, 0], [0, -10, -9, 0], [40, 0, 0, 5], [0, -3, -12, 0], [-22, 0, 0, -9], [0, 1, -15, 0], [9, 0, 0, -16], [25, 0, 0, -23], [0, 0, -12, 0], [0, -10, -29, 0], [0, -16, -13, 0], [3, 0, 0, 10], [-7, 0, 0, -4], [15, 0, 0, -12], [0, 9, -23, 0], [0, 24, 3, 0], [28, 0, 0, -14], [-27, 0, 0, -9], [0, 8, -18, 0], [0, -3, 3, 0], [13, 0, 0, -21], [0, -16, 5, 0], [0, -7, -5, 0], [3, 0, 0, 13], [0, -12, -26, 0], [0, -9, -15, 0], [0, -6, -34, 0], [-3, 0, 0, 2], [40, 0, 0, -14], [24, 0, 0, -9], [0, 30, 14, 0], [9, 0, 0, -13], [0, 7, -20, 0], [0, -2, -11, 0], [0, 4, -33, 0], [6, 0, 0, -26], [0, -6, 30, 0], [0, -3, -33, 0], [-16, 0, 0, -2], [0, -9, -37, 0], [37, 0, 0, 6], [0, -14, -14, 0], [18, 0, 0, 13], [-10, 0, 0, 8], [-36, 0, 0, 16], [0, -3, 34, 0], [-2, 0, 0, 25], [24, 0, 0, 0], [2, 0, 0, -15], [27, 0, 0, -4], [0, 31, 14, 0], [43, 0, 0, -7], [0, 16, 21, 0], [12, 0, 0, 12], [0, -5, -13, 0], [0, -19, 17, 0], [0, 14, 17, 0], [-15, 0, 0, 11], [0, -12, -49, 0], [0, -2, 8, 0], [-2, 0, 0, -16], [6, 0, 0, 21], [0, -2, -13, 0], [0, 14, 8, 0], [-25, 0, 0, -6], [27, 0, 0, -18], [0, 2, 35, 0], [0, -4, 50, 0], [11, 0, 0, 23], [0, 19, -13, 0], [0, -25, -25, 0], [0, -22, -13, 0], [30, 0, 0, 2], [-22, 0, 0, -7], [0, -14, 30, 0], [7, 0, 0, 8], [24, 0, 0, -31], [0, 18, 21, 0], [13, 0, 0, -24], [0, -3, 54, 0], [-9, 0, 0, 3], [8, 0, 0, -12], [0, 12, -28, 0], [15, 0, 0, 3], [0, -21, 1, 0], [0, 6, 33, 0], [3, 0, 0, 18], [-4, 0, 0, -19], [0, 7, 15, 0], [39, 0, 0, -3], [0, -12, -54, 0], [0, -12, 15, 0], [-49, 0, 0, -10], [-20, 0, 0, -3], [-6, 0, 0, -6], [0, 10, 2, 0], [0, 3, -60, 0], [-11, 0, 0, 8], [0, 17, -13, 0], [18, 0, 0, 9], [0, -20, -6, 0], [21, 0, 0, -12], [0, -14, 11, 0], [49, 0, 0, -10], [-25, 0, 0, -22], [18, 0, 0, 20], [0, 0, 55, 0], [0, 9, -51, 0], [7, 0, 0, 10], [0, -13, 20, 0], [9, 0, 0, -18], [6, 0, 0, 4], [0, -9, 3, 0], [8, 0, 0, 6], [0, 19, 29, 0], [-20, 0, 0, -10], [0, 30, 15, 0], [-12, 0, 0, 8], [0, 11, -23, 0], [12, 0, 0, 14], [10, 0, 0, -9], [0, -17, -29, 0], [0, 16, -3, 0], [-60, 0, 0, 2], [0, 4, -14, 0], [-38, 0, 0, 13], [0, -15, 18, 0], [33, 0, 0, 6], [17, 0, 0, -27], [0, 1, 3, 0], [0, 2, 38, 0], [0, -17, 15, 0], [0, -20, -28, 0], [0, 0, 0, 0], [34, 0, 0, 4], [0, 5, -9, 0], [0, 16, 31, 0], [-17, 0, 0, 15], [39, 0, 0, -22], [0, 0, 0, -27], [0, 16, 41, 0], [0, 11, 62, 0], [23, 0, 0, 12], [0, -16, -8, 0], [0, -23, 8, 0], [-56, 0, 0, 9], [0, -12, 32, 0], [0, 5, -24, 0], [0, 34, 22, 0], [19, 0, 0, -41], [-10, 0, 0, -8], [21, 0, 0, 8], [0, 14, 26, 0], [5, 0, 0, 6], [0, -10, -34, 0], [35, 0, 0, -2], [0, 14, -23, 0], [-51, 0, 0, -3], [0, -27, -20, 0], [0, -33, -18, 0], [19, 0, 0, -33], [6, 0, 0, 7], [30, 0, 0, 1], [0, 24, 36, 0], [0, -2, 13, 0], [-15, 0, 0, 33], [0, 6, -62, 0], [-54, 0, 0, 16], [-19, 0, 0, 26], [0, -21, 15, 0], [9, 0, 0, 24], [0, 12, 11, 0], [0, 3, 43, 0], [0, 0, 69, 0], [19, 0, 0, -8], [0, -4, 29, 0], [-10, 0, 0, -17], [0, -11, -15, 0], [0, 8, 14, 0], [-38, 0, 0, -12], [-60, 0, 0, 1], [0, 16, 26, 0], [0, 7, -23, 0], [18, 0, 0, -10], [24, 0, 0, -27], [0, -22, -24, 0], [0, -18, -18, 0], [-65, 0, 0, -6], [-15, 0, 0, 6], [-21, 0, 0, -15], [0, 10, 29, 0], [42, 0, 0, 10], [20, 0, 0, -8], [0, 3, 17, 0], [-54, 0, 0, 24], [0, -4, 42, 0], [0, -28, 14, 0], [-13, 0, 0, -8], [16, 0, 0, 11], [0, -19, 18, 0], [0, -25, -43, 0], [0, 19, 7, 0], [-7, 0, 0, 32], [0, 13, 52, 0], [-29, 0, 0, 24], [0, 12, -6, 0], [-13, 0, 0, 14], [0, -19, 23, 0], [-8, 0, 0, 27], [0, 37, 31, 0], [26, 0, 0, 4], [0, -31, -14, 0], [0, 10, -34, 0], [0, -33, -24, 0], [9, 0, 0, 12], [5, 0, 0, 8], [0, -1, -46, 0], [12, 0, 0, -28], [38, 0, 0, 7], [0, -25, -56, 0], [-27, 0, 0, -11], [0, 27, 3, 0], [-40, 0, 0, 0], [0, -9, -7, 0], [-75, 0, 0, 6], [0, 36, -5, 0], [34, 0, 0, -12], [0, -3, -78, 0], [30, 0, 0, -6], [3, 0, 0, -30], [0, 3, 45, 0], [0, 8, -13, 0], [0, -15, 41, 0], [33, 0, 0, 26], [0, 47, 35, 0], [-21, 0, 0, 4], [0, 7, 48, 0], [0, -26, 32, 0], [0, 12, 3, 0], [0, -18, -29, 0], [-16, 0, 0, -3], [63, 0, 0, -13], [10, 0, 0, -14], [0, 24, -18, 0], [12, 0, 0, -13], [47, 0, 0, -4], [0, 5, -11, 0], [18, 0, 0, -19], [-3, 0, 0, -12], [0, -44, -34, 0], [60, 0, 0, -9], [0, 19, 27, 0], [15, 0, 0, 12], [0, -27, 21, 0], [0, 32, 58, 0], [0, 23, -40, 0], [-5, 0, 0, 33], [0, 39, 18, 0], [-37, 0, 0, 34], [0, -10, -44, 0], [0, -4, 9, 0], [0, 36, 18, 0], [-98, 0, 0, 1], [0, -34, -40, 0], [-6, 0, 0, -10], [0, -28, -47, 0], [45, 0, 0, 0], [0, 10, -34, 0], [40, 0, 0, 12], [-54, 0, 0, 0], [-91, 0, 0, 4], [57, 0, 0, 12], [-32, 0, 0, -15], [0, 2, -7, 0], [0, -2, -27, 0], [0, -11, -59, 0], [-81, 0, 0, -2], [8, 0, 0, -4], [0, 12, 14, 0], [15, 0, 0, -44], [0, -18, -2, 0], [84, 0, 0, -11], [0, 1, 20, 0], [0, -13, 53, 0], [0, 34, 40, 0], [-25, 0, 0, 23], [0, 29, 58, 0], [-63, 0, 0, 4], [33, 0, 0, -17], [0, -41, -27, 0], [0, -18, -33, 0], [0, 2, 20, 0], [-57, 0, 0, 24], [0, 20, 46, 0], [0, -36, -18, 0], [-15, 0, 0, 32], [0, 4, -49, 0], [0, 16, 10, 0], [0, 19, -23, 0], [-48, 0, 0, 13], [-40, 0, 0, -10], [-18, 0, 0, 4]]
